@@ -4,6 +4,7 @@ from datetime import datetime
 import requests
 from os import path
 
+
 def json_parameters_function():
     def inner(func):
         def wrapper(json_object):
@@ -234,8 +235,70 @@ class JsonObject:
         return None
 
 
-class JsonList(list):
+class SortOrder:
+    Ascending = 0
+    Descending = 1
 
+
+class SearchType:
+    Aprox = 0
+    Exact = 1
+
+
+def bin_search(l, v, order=SortOrder.Ascending, search_type=SearchType.Aprox, key=None):
+    lo = 0
+    hi = len(l) - 1
+    mi = int((hi + lo) / 2)
+    if order == SortOrder.Ascending:
+        if key is None:
+            while hi != lo:
+                if l[mi] == v:
+                    return mi
+                if v > l[mi]:
+                    lo = mi + 1
+                else:
+                    hi = mi
+                mi = (hi + lo) // 2
+        else:
+            cv = key(l[mi])
+            while hi != lo:
+                if cv == v:
+                    return mi
+                if v > cv:
+                    lo = mi + 1
+                else:
+                    hi = mi
+                mi = (hi + lo) // 2
+                cv = key(l[mi])
+    else:
+        if key is None:
+            while hi != lo:
+                if l[mi] == v:
+                    return mi
+                if v < l[mi]:
+                    lo = mi + 1
+                else:
+                    hi = mi
+                mi = (hi + lo) // 2
+        else:
+            cv = key(l[mi])
+            while hi != lo:
+                if cv == v:
+                    return mi
+                if v < cv:
+                    lo = mi + 1
+                else:
+                    hi = mi
+                mi = (hi + lo) // 2
+                cv = key(l[mi])
+
+    if search_type == SearchType.Aprox:
+        return mi
+    else:
+        raise RuntimeError("Value not found")
+
+
+class JsonList(list):
     def __init__(self, list_type=None, iterable=None, allow_empty: bool = False):
         iterable = list() if not iterable else iterable
         iter(iterable)
@@ -318,7 +381,7 @@ class JsonList(list):
                 l.append(vars(i)[m])
         return l
 
-    def where(self, m: str, v, o: str="=="):
+    def where(self, m: str, v, o: str = "=="):
         d = {}
         if type(v) is str:
             criteria = "def criteria(i): return i.%s %s '%s'" % (m, o, v)
@@ -331,7 +394,7 @@ class JsonList(list):
         return self.filter(d["criteria"])
 
     def split_by(self, m) -> dict:
-        if type(m) is str and issubclass(self.list_type, JsonObject) :
+        if type(m) is str and issubclass(self.list_type, JsonObject):
             d = {}
             exec("def criteria(i): return i.%s" % m, d)
             m = d["criteria"]
@@ -344,30 +407,27 @@ class JsonList(list):
             r[l].append(i)
         return r
 
-    def filter(self, l):
+    def filter(self, key):
         nl = self.__class__()
         for i in self:
-            if l(i):
+            if key(i):
                 nl.append(i)
         return nl
 
-    def find_first(self, l):
-        return self[self.find_first_index(l)]
+    def find_first(self, key):
+        return self[self.find_first_index(key)]
 
-    def find_first_index(self, l):
+    def find_first_index(self, key):
         for ix, i in enumerate(self):
-            if l(i):
+            if key(i):
                 return ix
         raise RuntimeError("Value not found")
 
-    def find_ordered(self, l, value):
-        return self[self.find_ordered_index(l, value)]
+    def find_ordered(self, value, key=None, search_type=SearchType.Exact):
+        return self[self.find_ordered_index(value, key=key, search_type=search_type)]
 
-    def find_ordered_index(self, l, value):
-        from bisect import bisect_left
-        i = bisect_left(self, value, key=l)
-        if i < 0 or i >= len(self) or l(self[i]) != value:
-            raise RuntimeError("Value %s not found" % str(value))
+    def find_ordered_index(self, value, key=None, search_type=SearchType.Exact):
+        i = bin_search(self, value, key=key, search_type=search_type)
         return i
 
     def process(self, l):
